@@ -23,7 +23,10 @@ import os
 import shlex
 import sys
 
-from diffuse import constants  # type: ignore
+from gettext import gettext as _
+from typing import Any, Dict, Final, List, Optional, Tuple
+
+from diffuse import constants
 from diffuse import utils
 
 import gi  # type: ignore
@@ -33,16 +36,16 @@ from gi.repository import Gtk  # type: ignore # noqa: E402
 
 # class to store preferences and construct a dialogue for manipulating them
 class Preferences:
-    def __init__(self, path):
-        self.bool_prefs = {}
-        self.int_prefs = {}
-        self.string_prefs = {}
-        self.int_prefs_min = {}
-        self.int_prefs_max = {}
-        self.string_prefs_enums = {}
+    def __init__(self, path: str) -> None:
+        self.path = path
+        self.bool_prefs: Dict[str, bool] = {}
+        self.string_prefs: Dict[str, str] = {}
+        self.int_prefs: Dict[str, int] = {}
+        self.int_prefs_min: Dict[str, int] = {}
+        self.int_prefs_max: Dict[str, int] = {}
 
         # find available encodings
-        self.encodings = sorted(set(encodings.aliases.aliases.values()))
+        self.encodings: List[Optional[str]] = sorted(set(encodings.aliases.aliases.values()))
 
         if utils.isWindows():
             svk_bin = 'svk.bat'
@@ -51,7 +54,7 @@ class Preferences:
 
         auto_detect_codecs = ['utf_8', 'utf_16', 'latin_1']
         e = utils.norm_encoding(sys.getfilesystemencoding())
-        if e not in auto_detect_codecs:
+        if e is not None and e not in auto_detect_codecs:
             # insert after UTF-8 as the default encoding may prevent UTF-8 from
             # being tried
             auto_detect_codecs.insert(2, e)
@@ -121,7 +124,7 @@ class Preferences:
         ]
 
         # conditions used to determine if a preference should be greyed out
-        self.disable_when = {
+        self.disable_when: Final[Dict[str, Tuple[str, bool]]] = {
             'display_right_margin': ('display_show_right_margin', False),
             'display_ignore_whitespace_changes': ('display_ignore_whitespace', True),
             'display_ignore_blanklines': ('display_ignore_whitespace', True),
@@ -162,9 +165,9 @@ class Preferences:
                 _('Version control system search order')
             ]
         ]
-        vcs_folders_template = ['FolderSet']
+        vcs_folders_template: List[Any] = ['FolderSet']
         for key, name, cmd in vcs:
-            temp = ['List']
+            temp: List[Any] = ['List']
             if key == 'rcs':
                 # RCS uses multiple commands
                 temp.extend([['File', key + '_bin_co', 'co', _('"co" command')],
@@ -193,11 +196,11 @@ class Preferences:
         self.default_bool_prefs = self.bool_prefs.copy()
         self.default_int_prefs = self.int_prefs.copy()
         self.default_string_prefs = self.string_prefs.copy()
+
         # load the user's preferences
-        self.path = path
         if os.path.isfile(self.path):
             try:
-                with open(self.path, 'r',  encoding='utf-8') as f:
+                with open(self.path, 'r', encoding='utf-8') as f:
                     ss = utils.readconfiglines(f)
                 for j, s in enumerate(ss):
                     try:
@@ -205,13 +208,13 @@ class Preferences:
                         if len(a) > 0:
                             p = a[0]
                             if len(a) == 2 and p in self.bool_prefs:
-                                self.bool_prefs[p] = (a[1] == 'True')
+                                self.setBool(p, a[1] == 'True')
                             elif len(a) == 2 and p in self.int_prefs:
-                                self.int_prefs[p] = max(
+                                self.setInt(p, max(
                                     self.int_prefs_min[p],
-                                    min(int(a[1]), self.int_prefs_max[p]))
+                                    min(int(a[1]), self.int_prefs_max[p])))
                             elif len(a) == 2 and p in self.string_prefs:
-                                self.string_prefs[p] = a[1]
+                                self.setString(p, a[1])
                             else:
                                 raise ValueError()
                     except ValueError:
@@ -234,13 +237,13 @@ class Preferences:
                 self._initFromTemplate(template[i])
                 i += 1
         elif template[0] == 'Boolean':
-            self.bool_prefs[template[1]] = template[2]
+            self.setBool(template[1], template[2])
         elif template[0] == 'Integer':
-            self.int_prefs[template[1]] = template[2]
+            self.setInt(template[1], template[2])
             self.int_prefs_min[template[1]] = template[4]
             self.int_prefs_max[template[1]] = template[5]
         elif template[0] in ['String', 'File', 'Font', 'Encoding']:
-            self.string_prefs[template[1]] = template[2]
+            self.setString(template[1], template[2])
 
     # callback used when a preference is toggled
     def _toggled_cb(self, widget, widgets, name):
@@ -252,16 +255,16 @@ class Preferences:
 
     # display the dialogue and update the preference values if the accept
     # button was pressed
-    def runDialog(self, parent):
+    def runDialog(self, parent: Gtk.Widget) -> None:
         dialog = Gtk.Dialog(_('Preferences'), parent=parent, destroy_with_parent=True)
-        dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT)
-        dialog.add_button(Gtk.STOCK_OK, Gtk.ResponseType.OK)
+        dialog.add_button(_('_Cancel'), Gtk.ResponseType.REJECT)
+        dialog.add_button(_('_OK'), Gtk.ResponseType.OK)
 
-        widgets = {}
+        widgets: Dict[str, Gtk.Widget] = {}
         w = self._buildPrefsDialog(parent, widgets, self.template)
         # disable any preferences than are not relevant
-        for k, v in self.disable_when.items():
-            p, t = v
+        for k, tuple_value in self.disable_when.items():
+            p, t = tuple_value
             if widgets[p].get_active() == t:
                 widgets[k].set_sensitive(False)
         dialog.vbox.add(w)
@@ -270,22 +273,23 @@ class Preferences:
         accept = (dialog.run() == Gtk.ResponseType.OK)
         if accept:
             for k in self.bool_prefs:
-                self.bool_prefs[k] = widgets[k].get_active()
+                self.setBool(k, widgets[k].get_active())
             for k in self.int_prefs:
-                self.int_prefs[k] = widgets[k].get_value_as_int()
+                self.setInt(k, widgets[k].get_value_as_int())
             for k in self.string_prefs:
-                self.string_prefs[k] = utils.null_to_empty(widgets[k].get_text())
+                text = self._getWidgetText(widgets[k])
+                self.setString(k, utils.null_to_empty(text))
             try:
                 ss = []
-                for k, v in self.bool_prefs.items():
-                    if v != self.default_bool_prefs[k]:
-                        ss.append(f'{k} {v}\n')
-                for k, v in self.int_prefs.items():
-                    if v != self.default_int_prefs[k]:
-                        ss.append(f'{k} {v}\n')
-                for k, v in self.string_prefs.items():
-                    if v != self.default_string_prefs[k]:
-                        v_escaped = v.replace('\\', '\\\\').replace('"', '\\"')
+                for k, bool_value in self.bool_prefs.items():
+                    if bool_value != self.default_bool_prefs[k]:
+                        ss.append(f'{k} {bool_value}\n')
+                for k, int_value in self.int_prefs.items():
+                    if int_value != self.default_int_prefs[k]:
+                        ss.append(f'{k} {int_value}\n')
+                for k, str_value in self.string_prefs.items():
+                    if str_value != self.default_string_prefs[k]:
+                        v_escaped = str_value.replace('\\', '\\\\').replace('"', '\\"')
                         ss.append(f'{k} "{v_escaped}"\n')
                 ss.sort()
                 with open(self.path, 'w', encoding='utf-8') as f:
@@ -303,11 +307,11 @@ class Preferences:
     def _buildPrefsDialog(self, parent, widgets, template):
         tpl_section = template[0]
         if tpl_section == 'FolderSet':
-            notebook = Gtk.Notebook.new()
+            notebook = Gtk.Notebook()
             notebook.set_border_width(10)
             i = 1
             while i < len(template):
-                label = Gtk.Label.new(template[i])
+                label = Gtk.Label(label=template[i])
                 i += 1
                 w = self._buildPrefsDialog(parent, widgets, template[i])
                 i += 1
@@ -316,7 +320,7 @@ class Preferences:
                 label.show()
             return notebook
 
-        table = Gtk.Grid.new()
+        table = Gtk.Grid()
         table.set_border_width(10)
         for i, tpl in enumerate(template[1:]):
             tpl_section = tpl[0]
@@ -326,69 +330,93 @@ class Preferences:
                 w.show()
             elif tpl_section == 'Boolean':
                 button = Gtk.CheckButton.new_with_mnemonic(tpl[3])
-                button.set_active(self.bool_prefs[tpl[1]])
+                button.set_active(self.getBool(tpl[1]))
                 widgets[tpl[1]] = button
                 table.attach(button, 1, i, 1, 1)
                 button.connect('toggled', self._toggled_cb, widgets, tpl[1])
                 button.show()
             else:
-                label = Gtk.Label.new(tpl[3] + ': ')
+                label = Gtk.Label(label=tpl[3] + ': ')
                 label.set_xalign(1.0)
                 label.set_yalign(0.5)
                 table.attach(label, 0, i, 1, 1)
                 label.show()
-                if tpl[0] in ['Font', 'Integer']:
-                    entry = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
-                    if tpl[0] == 'Font':
-                        button = _FontButton()
-                        button.set_font(self.string_prefs[tpl[1]])
+                if tpl_section in ['Font', 'Integer']:
+                    entry = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
+                    if tpl_section == 'Font':
+                        button = Gtk.FontButton()
+                        button.set_font(self.getString(tpl[1]))
                     else:
-                        button = Gtk.SpinButton.new(
-                            Gtk.Adjustment.new(self.int_prefs[tpl[1]], tpl[4], tpl[5], 1, 0, 0),
-                            1.0,
-                            0)
+                        adj = Gtk.Adjustment(
+                            value=self.getInt(tpl[1]),
+                            lower=tpl[4],
+                            upper=tpl[5],
+                            step_increment=1,
+                            page_increment=0,
+                            page_size=0)
+                        button = Gtk.SpinButton(
+                            adjustment=adj,
+                            climb_rate=1.0,
+                            digits=0)
                     widgets[tpl[1]] = button
                     entry.pack_start(button, False, False, 0)
                     button.show()
                 else:
-                    if tpl[0] == 'Encoding':
+                    if tpl_section == 'Encoding':
                         entry = utils.EncodingMenu(self)
                         entry.set_text(tpl[3])
-                    elif tpl[0] == 'File':
+                    elif tpl_section == 'File':
                         entry = _FileEntry(parent, tpl[3])
                     else:
-                        entry = Gtk.Entry.new()
+                        entry = Gtk.Entry()
                     widgets[tpl[1]] = entry
-                    entry.set_text(self.string_prefs[tpl[1]])
+                    entry.set_text(self.getString(tpl[1]))
                 table.attach(entry, 1, i, 1, 1)
                 entry.show()
             table.show()
         return table
 
+    def _getWidgetText(self, widget):
+        text = ""
+        if (
+            isinstance(widget, Gtk.Entry) or
+            isinstance(widget, utils.EncodingMenu) or
+            isinstance(widget, _FileEntry)
+        ):
+            text = widget.get_text()
+        elif isinstance(widget, Gtk.FontButton):
+            text = widget.get_font()
+        else:
+            raise TypeError(f"Don't know how to get text from type: {type(widget)}")
+        return text
+
     # get/set methods to manipulate the preference values
-    def getBool(self, name):
+    def getBool(self, name: str) -> bool:
         return self.bool_prefs[name]
 
-    def setBool(self, name, value):
+    def setBool(self, name: str, value: bool) -> None:
         self.bool_prefs[name] = value
 
-    def getInt(self, name):
+    def getInt(self, name: str) -> int:
         return self.int_prefs[name]
 
-    def getString(self, name):
+    def setInt(self, name: str, value: int) -> None:
+        self.int_prefs[name] = value
+
+    def getString(self, name: str) -> str:
         return self.string_prefs[name]
 
-    def setString(self, name, value):
+    def setString(self, name: str, value: str) -> None:
         self.string_prefs[name] = value
 
-    def getEncodings(self):
+    def getEncodings(self) -> List[Optional[str]]:
         return self.encodings
 
-    def _getDefaultEncodings(self):
-        return self.string_prefs['encoding_auto_detect_codecs'].split()
+    def _getDefaultEncodings(self) -> List[str]:
+        return self.getString('encoding_auto_detect_codecs').split()
 
-    def getDefaultEncoding(self):
-        return self.string_prefs['encoding_default_codec']
+    def getDefaultEncoding(self) -> str:
+        return self.getString('encoding_default_codec')
 
     # attempt to convert a string to unicode from an unknown encoding
     def convertToUnicode(self, s):
@@ -410,7 +438,7 @@ class Preferences:
 
     # cygwin and native applications can be used on windows, use this method
     # to convert a path to the usual form expected on sys.platform
-    def convertToNativePath(self, s):
+    def convertToNativePath(self, s: str) -> str:
         if utils.isWindows() and s.find('/') >= 0:
             # treat as a cygwin path
             s = s.replace(os.sep, '/')
@@ -434,27 +462,18 @@ class Preferences:
         return s
 
 
-# adaptor class to allow a Gtk.FontButton to be read like a Gtk.Entry
-class _FontButton(Gtk.FontButton):
-    def __init__(self):
-        Gtk.FontButton.__init__(self)
-
-    def get_text(self):
-        return self.get_font_name()
-
-
 # text entry widget with a button to help pick file names
 class _FileEntry(Gtk.Box):
-    def __init__(self, parent, title):
+    def __init__(self, parent: Gtk.Widget, title: str) -> None:
         Gtk.Box.__init__(self, orientation=Gtk.Orientation.HORIZONTAL)
         self.toplevel = parent
         self.title = title
-        self.entry = entry = Gtk.Entry.new()
+        self.entry = entry = Gtk.Entry()
         self.pack_start(entry, True, True, 0)
         entry.show()
-        button = Gtk.Button.new()
-        image = Gtk.Image.new()
-        image.set_from_stock(Gtk.STOCK_OPEN, Gtk.IconSize.MENU)
+        button = Gtk.Button()
+        image = Gtk.Image()
+        image.set_from_icon_name('document-open-symbolic', Gtk.IconSize.MENU)
         button.add(image)
         image.show()
         button.connect('clicked', self.chooseFile)
@@ -462,19 +481,19 @@ class _FileEntry(Gtk.Box):
         button.show()
 
     # action performed when the pick file button is pressed
-    def chooseFile(self, widget):
+    def chooseFile(self, widget: Gtk.Widget) -> None:
         dialog = Gtk.FileChooserDialog(
             self.title,
             self.toplevel,
             Gtk.FileChooserAction.OPEN,
-            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+            (_('_Cancel'), Gtk.ResponseType.CANCEL, _('_Open'), Gtk.ResponseType.OK))
         dialog.set_current_folder(os.path.realpath(os.curdir))
         if dialog.run() == Gtk.ResponseType.OK:
             self.entry.set_text(dialog.get_filename())
         dialog.destroy()
 
-    def set_text(self, s):
+    def set_text(self, s: str) -> None:
         self.entry.set_text(s)
 
-    def get_text(self):
+    def get_text(self) -> str:
         return self.entry.get_text()
